@@ -44,10 +44,21 @@ PIXEL_SHIFT =       0x05
 SHIFT_FORWARD =     0x20
 SHIFT_BACKWARD =    0x00
 
+PING_READ =         0x75
+SYSTEM_RESET =      0xFF
+
+
 class __pyInterfaz(Board):
 
-    def __init__(self, com_port, baudrate=57600, layout=None):
+    def __init__(self, com_port, baudrate=115200, layout=None):
         super().__init__(com_port,  baudrate=baudrate, layout=layout)
+        # IDENTIFICAMOS EL MODELO
+        self.add_cmd_handler(pyfirmata.REPORT_FIRMWARE, self._handle_report_firmware)
+        self.send_sysex(pyfirmata.QUERY_FIRMWARE, []);
+        self.pass_time(0.1)  # Serial SYNC
+        while self.bytes_available():
+            self.iterate();
+        self.load_board()
         # Necesitamos escribir directo sobre el puerto e indicamos conexión
         if self.led_builtin:
             self.sp.write([SET_DIGITAL_PIN,self.led_builtin,HIGH]);
@@ -57,7 +68,34 @@ class __pyInterfaz(Board):
         it = util.Iterator(self)
         it.start()
         self.add_cmd_handler(I2C_REPLY, self._handle_i2c_message)
+        self.add_cmd_handler(PING_READ, self._handle_ping_read)
         self.send_sysex(I2C_CONFIG, []);  # I2C_CONFIG
+        self.lastMsg = ''
+
+    def load_board(self):
+        if self.firmware is not None:
+            if hasattr(self, self.firmware):
+                b = getattr(self, self.firmware)
+                b()
+                return
+        # Si no podemos identificar por firmware lo hacemos por hardware
+        m = None
+        if len(self._layout['digital']) == 18 and len(self._layout['analog']) == 6: # Arduino UNO
+            m = 'uno'
+        elif len(self._layout['digital']) == 18 and len(self._layout['analog']) == 8: # Arduino Nano
+            m = 'rasti'
+        if m is not None and hasattr(self, m):
+            b = getattr(self, m)
+            b()
+        else:
+            raise ValueError("No se pudo identificar el modelo")
+
+    def send_command(self, cmd):
+        msg = bytearray([cmd])
+        self.sp.write(msg)
+
+    def reset(self):
+        self.send_command(SYSTEM_RESET)
 
     def _handle_i2c_message(self, *args, **kwargs):
         address = util.from_two_bytes([args[0], args[1]])
@@ -85,29 +123,129 @@ class __pyInterfaz(Board):
         except IndexError:
             raise ValueError
 
+    def _handle_ping_read(self, pin_nr, *data):
+        duration = (util.from_two_bytes(data[1:3]) << 24) \
+                    + (util.from_two_bytes(data[3:5]) << 16) \
+                    + (util.from_two_bytes(data[5:7]) << 8) \
+                    + util.from_two_bytes(data[7:9])
+        ping_distance = duration / 58.2
+        # We can calculate the distance using an HC-SR04 as 'duration / 58.2'
+        try:
+            ## CALL CALLBACK
+            if self.digital[pin_nr].ping_distance != ping_distance:
+                for a in self._pings:
+                    if a.index == pin_nr:
+                        a._changecb(ping_distance)
+            self.digital[pin_nr].ping_distance = ping_distance
+        except IndexError:
+            raise ValueError
+
+    def _handle_report_capability_response(self, *data):
+        charbuffer = []
+        pin_spec_list = []
+
+        for c in data:
+            if c == pyfirmata.CAPABILITY_RESPONSE:
+                continue
+
+            charbuffer.append(c)
+            if c == 0x7F:
+                # A copy of charbuffer
+                pin_spec_list.append(charbuffer[:])
+                charbuffer = []
+
+        self._layout = self.pin_list_to_board_dict(pin_spec_list)
+
+    def pin_list_to_board_dict(self, pinlist):
+        """
+        Capability Response codes:
+            INPUT:  0, 1
+            OUTPUT: 1, 1
+            ANALOG: 2, 10
+            PWM:    3, 8
+            SERV0:  4, 14
+            I2C:    6, 1
+            PULLUP: 11, 1
+        """
+
+        board_dict = {
+            "digital": [],
+            "analog": [],
+            "pwm": [],
+            "servo": [],  # 2.2 specs
+            # 'i2c': [],  # 2.3 specs
+            "disabled": [],
+        }
+
+        for i, pin in enumerate(pinlist):
+            pin.pop()  # removes the 0x79 on end
+            if not pin:
+                board_dict["disabled"] += [i]
+                #board_dict["digital"] += [i]
+                continue
+
+            capabilities = pin[::2]
+            # Iterate over evens
+            if 0 in capabilities and 1 in capabilities:
+                board_dict["digital"] += [i]
+            if 2 in capabilities:
+                board_dict["analog"] += [i]
+            if 3 in capabilities:
+                board_dict["pwm"] += [i]
+            if 4 in capabilities:
+                board_dict["servo"] += [i]
+            # Desable I2C
+            if 6 in capabilities:
+                pass
+
+        # We have to deal with analog pins:
+        # - (14, 15, 16, 17, 18, 19)
+        # + (0, 1, 2, 3, 4, 5)
+        diff = set(board_dict["digital"]) - set(board_dict["analog"])
+        board_dict["analog"] = [n for n, _ in enumerate(board_dict["analog"])]
+
+        # Digital pin problems:
+        # - (2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15)
+        # + (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13)
+        #board_dict["digital"] = [n for n, _ in enumerate(diff)]
+        # Based on lib Arduino 0017
+        #board_dict["servo"] = board_dict["digital"]
+        board_dict["servo"] = [n for n, _ in enumerate(diff)]
+
+        # Turn lists into tuples
+        # Using dict for Python 2.6 compatibility
+        board_dict = dict([(key, tuple(value)) for key, value in board_dict.items()])
+
+        print(board_dict)
+        return board_dict
+
     def output(self, index):
-        if index < 1: index = 1
-        return self._outputs[index - 1]
+        if index < 0: index = 0
+        return self._outputs[index]
 
     def input(self, index):
-        if index < 1: index = 1
-        return self._analogs[index - 1]
+        if index < 0: index = 0
+        return self._analogs[index]
 
     def digital_input(self, index):
-        if index < 1: index = 1
-        return self._digitals[index - 1]
+        if index < 0: index = 0
+        return self._digitals[index]
 
     def servo(self, index):
-        if index < 1: index = 1
-        return self._servos[index - 1]
+        if index < 0: index = 0
+        return self._servos[index]
 
     def pin(self, index):
-        if index < 1: index = 1
-        return self._pins[index - 1]
+        if index < 0: index = 0
+        return self._pins[index]
+
+    def ping(self, index):
+        if index < 0: index = 0
+        return self._pings[index]
 
     def pixel(self, index):
-        if index < 1: index = 1
-        return self._pixels[index - 1]
+        if index < 0: index = 0
+        return self._pixels[index]
 
     def i2c(self, address):
         if not address in self._i2c: self._i2c[address] = self._I2C(self, address);
@@ -151,26 +289,37 @@ class __pyInterfaz(Board):
             return buf
 
         def push(self, str):
+            if self._silenciado:
+                return  self;
             data = [CMD_LCD_PUSH]
             data += self._strtosysex(str)
             self._interfaz.send_sysex(CMD_LCD_DATA, data)
             return self
 
-        def print(self, row, str):
+        def write(self, row, str):
             data = [CMD_LCD_PRINT, row]
             data += self._strtosysex(str)
             self._interfaz.send_sysex(CMD_LCD_DATA, data)
             return self
 
+        def print(self, row, str):
+            if self._silenciado:
+                return  self;
+            self.write(row,str);
+            return self
+
         def clear(self):
             self._interfaz.send_sysex(CMD_LCD_DATA, [CMD_LCD_CLEAR])
             return self
 
         def silence(self):
+            self._interfaz.print("LCD", "silenciado")
             self._silenciado = True
+            return self
 
         def on(self):
             self._silenciado = False
+            self._interfaz.print("LCD", "encendido")
             return self
 
     class _Servo:
@@ -216,7 +365,7 @@ class __pyInterfaz(Board):
             self._interfaz.print("salida " + str(self.index + 1), "direccion " + str(d))
             return self
 
-        def speed(self, speed):
+        def power(self, speed):
             if speed > 100: speed = 100
             if speed < 0: speed = 0
             self._interfaz.send_sysex(CMD_MOTOR_DATA, [CMD_MOTOR_SPEED, self.index, speed & FIRMATA_7BIT_MASK, speed >> 7 & FIRMATA_7BIT_MASK])
@@ -328,6 +477,74 @@ class __pyInterfaz(Board):
             self.processCallback(callback)
             return self
 
+    class _PING(__Sensor):
+        def __init__(self, interfaz, index):
+            self._interfaz = interfaz
+            self.index = index + 14
+            self._interfaz.digital[self.index].ping_distance = -1;
+            super().__init__()
+
+        def ping(self):
+            self.exec_ping();
+
+        def exec_ping(self, trigger_mode=1, trigger_duration=10, echo_timeout=65000):
+            """
+            Trigger the pin and wait for a pulseIn echo.
+            Used with HC-SR04 ultrasonic ranging sensors
+            :arg trigger_mode: Uses value as a boolean,
+                            0 to trigger LOW,
+                            1 to trigger HIGH (default, for HC-SR04 modules).
+            :arg trigger_duration: Duration (us) for the trigger signal.
+            :arg echo_timeout: Time (us) to wait for the echo (pulseIn timeout).
+            """
+            if trigger_mode not in (0, 1):
+                raise IOError("trigger_mode should be 0 or 1")
+
+            # This is the protocol to ask for a pulseIn:
+            #       START_SYSEX(0xF0)           // send_sysex(...)
+            #       puseIn/pulseOut(0x74)       // send_sysex(PULSE_IN, ...)
+            #       pin(0-127)
+            #       value(1 or 0, HIGH or LOW)
+            #       pulseOutDuration 0 (LSB)
+            #       pulseOutDuration 0 (MSB)
+            #       pulseOutDuration 1 (LSB)
+            #       pulseOutDuration 1 (MSB)
+            #       pulseOutDuration 2 (LSB)
+            #       pulseOutDuration 2 (MSB)
+            #       pulseOutDuration 3 (LSB)
+            #       pulseOutDuration 3 (MSB)
+            #       pulseInTimeout 0 (LSB)
+            #       pulseInTimeout 0 (MSB)
+            #       pulseInTimeout 1 (LSB)
+            #       pulseInTimeout 1 (MSB)
+            #       pulseInTimeout 2 (LSB)
+            #       pulseInTimeout 2 (MSB)
+            #       pulseInTimeout 3 (LSB)
+            #       pulseInTimeout 3 (MSB)
+            #       END_SYSEX(0xF7)             // send_sysex(...)
+
+            data = bytearray()
+            data.append(self.index)  # Pin number
+            data.append(trigger_mode)  # Trigger mode (1 or 0, HIGH or LOW)
+            trigger_duration_arr = util.to_two_bytes((trigger_duration >> 24) & 0xFF) \
+                                   + util.to_two_bytes((trigger_duration >> 16) & 0xFF) \
+                                   + util.to_two_bytes((trigger_duration >> 8) & 0xFF) \
+                                   + util.to_two_bytes(trigger_duration & 0xFF)
+            data.extend(trigger_duration_arr)  # pulseOutDuration
+            echo_timeout_arr = util.to_two_bytes((echo_timeout >> 24) & 0xFF) \
+                               + util.to_two_bytes((echo_timeout >> 16) & 0xFF) \
+                               + util.to_two_bytes((echo_timeout >> 8) & 0xFF) \
+                               + util.to_two_bytes(echo_timeout & 0xFF)
+            data.extend(echo_timeout_arr)  # pulseInTimeout
+            self._interfaz.send_sysex(PING_READ, data)
+
+
+        def on(self, callback=None):
+            self.processCallback(callback)
+            self.ping()
+            self._interfaz.print("ultrasonido " + str(self.index - 14), "reportando")
+
+
     class _Analog(__Sensor):
         def __init__(self, interfaz, index):
             self._interfaz = interfaz
@@ -414,12 +631,12 @@ class __pyInterfaz(Board):
             colors = self.hex_to_rgb(hex)
             return (colors[0] << 16) + (colors[1] << 8) + (colors[2])
 
-        def config(self, length):
+        def create(self, length):
             self.length = length
             self._interfaz.digital[self.pin]._set_mode(pyfirmata.OUTPUT);
             buf = [PIXEL_CONFIG, self.pin, self.length & FIRMATA_7BIT_MASK, self.length >> 7 & FIRMATA_7BIT_MASK]
             self._interfaz.send_sysex(PIXEL_COMMAND, buf)
-            self._interfaz.print("pixel " + str(self.index), "configrado")
+            self._interfaz.print("pixel " + str(self.index), "configurado")
             return self
 
         def show(self):
@@ -451,7 +668,7 @@ class __pyInterfaz(Board):
         def on(self, pos=0):
             color = "#FFFFFF"
             if pos > 0:
-                self.pixel_color(color, pos)
+                self.color(color, pos)
             else:
                 self.strip_color(color)
             self._interfaz.print("pixel " + str(self.index), "encendido")
@@ -464,18 +681,41 @@ class __pyInterfaz(Board):
                 self.strip_color(color)
             self._interfaz.print("pixel " + str(self.index), "apagado")
 
+class interfaz(__pyInterfaz):
+    def __init__(self, com_port):
+        super().__init__(com_port, baudrate=115200, layout=None)
 
-class i32(__pyInterfaz):
-    def __init__(self, com_port, baudrate=115200):
-        self.boardlayout = {
-            'digital' : tuple(x for x in range(40)),
-            'analog' : tuple(x for x in range(20)),
-            'pwm' : tuple(x for x in range(40)),
-            'use_ports' : True,
-            'disabled' : (0, 1) # Rx, Tx
-        }    
+    def uno(self):
+        self.modelo = "Uno";
+        self.led_builtin = 13;
+        self._lcd = self._LCD(self)
+        self._outputs = [self._Output(self, 0), self._Output(self, 1), self._Output(self, 2), self._Output(self, 3)]
+        self._servos = [self._Servo(self, 1), self._Servo(self, 2)]
+        self._pins = [self._Pin(self, 1), self._Pin(self, 2)]
+        self._pixels = [self._Pixel(self, 1), self._Pixel(self, 2)]
+        self._analogs = [self._Analog(self, 0), self._Analog(self, 1), self._Analog(self, 2), self._Analog(self, 3)]
+        self._digitals = [self._Digital(self, 0), self._Digital(self, 1), self._Digital(self, 2), self._Digital(self, 3)]
+        self._pings = [self._PING(self, 0), self._PING(self, 1), self._PING(self, 2), self._PING(self, 3)]
+        self._i2c = dict()
+        self._joystick = self._Joystick(self)
+
+    def rasti(self):
+        self.led_builtin = 13;
+        self.modelo = "Rasti";
+        self._lcd = None
+        self._outputs = [self._Output(self, 0), self._Output(self, 1)]
+        self._servos = [self._Servo(self, 1), self._Servo(self, 2),self._Servo(self, 3), self._Servo(self, 4)]
+        self._analogs = [self._Analog(self, 0), self._Analog(self, 1), self._Analog(self, 2), self._Analog(self, 3)]
+        self._digitals = [self._Digital(self, 0), self._Digital(self, 1), self._Digital(self, 2), self._Digital(self, 3)]
+        self._pings = [self._PING(self, 0), self._PING(self, 1), self._PING(self, 2), self._PING(self, 3)]
+        self._i2c = dict()
+        self._lcd = None
+        self._joystick = self._Joystick(self)
+        self._pins = [self._Pin(self, 1), self._Pin(self, 2)]
+        self._pixels = [self._Pixel(self, 1), self._Pixel(self, 2)]
+
+    def i32(self):
         self.led_builtin = 2;
-        super().__init__(com_port, baudrate=baudrate, layout=self.boardlayout)
         self._lcd = self._LCD(self)
         self._outputs = [self._Output(self, 0), self._Output(self, 1), self._Output(self, 2), self._Output(self, 3)]
         self._servos = [self._Servo(self, 1), self._Servo(self, 2)]
@@ -486,36 +726,17 @@ class i32(__pyInterfaz):
         self._pixels = [self._Pixel(self, 1), self._Pixel(self, 2)]
 
 
-class interfaz(__pyInterfaz):
-    def __init__(self, com_port):
-        self.boardlayout = {
-            'digital': tuple(x for x in range(17)),
-            'analog': tuple(x for x in range(6)),
-            'pwm': (3, 5, 6, 9, 10, 11),
-            'use_ports' : True,
-            'disabled' : (0, 1) # Rx, Tx
-        }
-        self.led_builtin = 13;
-        super().__init__(com_port, baudrate=57600, layout=self.boardlayout)
-        self._lcd = self._LCD(self)
-        self._outputs = [self._Output(self, 0), self._Output(self, 1), self._Output(self, 2), self._Output(self, 3)]
-        self._servos = [self._Servo(self, 1), self._Servo(self, 2)]
-        self._pins = [self._Pin(self, 1), self._Pin(self, 2)]
-        self._pixels = [self._Pixel(self, 1), self._Pixel(self, 2)]
-        self._analogs = [self._Analog(self, 0), self._Analog(self, 1), self._Analog(self, 2), self._Analog(self, 3)]
-        self._digitals = [self._Digital(self, 0), self._Digital(self, 1), self._Digital(self, 2), self._Digital(self, 3)]
-        self._i2c = dict()
-        self._joystick = self._Joystick(self)
-
-class rasti(__pyInterfaz):
-    def __init__(self, com_port):
-        self.led_builtin = 13;
-        super().__init__(com_port)
-        self._lcd = None
-        self._outputs = [self._Output(self, 0), self._Output(self, 1)]
-        self._servos = [self._Servo(self, 1), self._Servo(self, 2)]
-        self._analogs = [self._Analog(self, 0), self._Analog(self, 1), self._Analog(self, 2), self._Analog(self, 3)]
-        self._i2c = dict()
-        self._joystick = self._Joystick(self)
-        self._pins = [self._Pin(self, 1), self._Pin(self, 2)]
-        self._pixels = [self._Pixel(self, 1), self._Pixel(self, 2)]
+"""
+def interfaz(com_port):
+    boardlayout = {'digital': (),'analog': (),'pwm': (),'use_ports': False,'disabled': ()}
+    b = Board(com_port, baudrate=115200, layout=None, timeout=2)
+    modelo = b.firmware
+    b.exit();
+    # DEVOLVER LA INSTANCIA CORRESPONDIENTE AL FIRMWARE
+    print(modelo)
+    if modelo == "uno":
+        return uno(com_port)
+    elif modelo == "rasti":
+        return rasti(com_port)
+    raise Exception("Error en modelo de interfaz")
+"""
